@@ -3,12 +3,44 @@ __license__ = "GPLv2"
 __maintainer__ = "Joseph Ryan"
 __email__ = "jr@aphyt.com"
 
+import collections
 import struct
 import copy
 from abc import ABC, abstractmethod
 import binascii
 
 
+def test_bit(int_type, offset):
+    mask = 1 << offset
+    if int_type & mask != 0:
+        response = True
+    else:
+        response = False
+    return response
+
+
+def set_bit(int_type, offset):
+    mask = 1 << offset
+    return int_type | mask
+
+
+def clear_bit(int_type, offset):
+    mask = ~(1 << offset)
+    return int_type & mask
+
+
+def toggle_bit(int_type, offset):
+    mask = 1 << offset
+    return int_type ^ mask
+
+
+def flatten(x):
+    if isinstance(x, collections.Iterable):
+        return [a for i in x for a in flatten(i)]
+    else:
+        return [x]
+
+
 class CIPDataType(ABC):
     """
     Abstract Base Class for CIP Data Types. The subclasses will be added to a data type dictionary and
@@ -462,6 +494,8 @@ class CIPStructure(CIPDataType):
                 offset = offset + (member_value.alignment - offset % member_value.alignment)
             end_byte = offset + member_value.size
             member_value.data = structure_data[offset:end_byte]
+            # Call value to handle nested structures
+            member_value.value()
             offset = end_byte
         return self
 
@@ -502,16 +536,36 @@ class CIPArray(CIPDataType):
     def alignment(self) -> int:
         return self._alignment
 
+    def _get_size(self) -> int:
+        """
+        Determine the amount of data required to send and receive data,
+        handling the special case of Boolean array packing
+        :return:
+        """
+        total_size = 1
+        last_number = None
+        for number in self.number_of_elements:
+            total_size = total_size * number
+            last_number = number
+        add_bytes = 0
+        if self.array_data_type == b'\xc1':
+            if last_number % 16 != 0:
+                add_bytes = 2
+            # print("Total %s size in bool %s" % (self.bytes(), total_size))
+            total_size = total_size // 8 + add_bytes
+            # print("Total %s size in bool %s" % (self.bytes(), total_size))
+            # self.array_data_type_size = required_words * 2
+        else:
+            total_size = total_size * self.array_data_type_size
+        return total_size
+
     def from_items(self, array_data_type, array_data_size, array_dimensions, number_of_elements, start_array_elements):
         self.array_data_type = array_data_type
         self.array_data_type_size = array_data_size
         self.array_dimensions = array_dimensions
         self.number_of_elements = number_of_elements
         self.start_array_elements = start_array_elements
-        total_size = self.array_data_type_size
-        for number in self.number_of_elements:
-            total_size = total_size * number
-        self.size = total_size
+        self.size = self._get_size()
         # ToDo calling get_class_type_data_code seems weird
         self._local_cip_data_type_object = _get_class_data_type_code(self.array_data_type)
         self._alignment = self._local_cip_data_type_object.alignment
@@ -523,10 +577,7 @@ class CIPArray(CIPDataType):
         self.array_dimensions = array_dimensions
         self.number_of_elements = number_of_elements
         self.start_array_elements = start_array_elements
-        total_size = self.array_data_type_size
-        for number in self.number_of_elements:
-            total_size = total_size * number
-        self.size = total_size
+        self.size = self._get_size()
         # ToDo calling get_class_type_data_code seems weird
         self._local_cip_data_type_object = cip_instance
         self._alignment = self._local_cip_data_type_object.alignment
@@ -543,9 +594,17 @@ class CIPArray(CIPDataType):
         if dimension == self.array_dimensions - 1:
             temp_array = []
             for element in range(self.number_of_elements[dimension]):
-                start_bytes = (position + element) * self.array_data_type_size
-                self._local_cip_data_type_object.data = \
-                    self.data[start_bytes:start_bytes + self.array_data_type_size]
+                if self.array_data_type == b'\xc1':
+                    start_bit = (position + element)
+                    if test_bit(int.from_bytes(self.data, 'little'), start_bit):
+                        integer_bool = 1
+                    else:
+                        integer_bool = 0
+                    self._local_cip_data_type_object.data = integer_bool.to_bytes(2, 'little')
+                else:
+                    start_bytes = (position + element) * self.array_data_type_size
+                    self._local_cip_data_type_object.data = \
+                        self.data[start_bytes:start_bytes + self.array_data_type_size]
                 temp_value = copy.deepcopy(self._local_cip_data_type_object)
                 temp_array.append(temp_value.value())
             return temp_array
@@ -559,6 +618,7 @@ class CIPArray(CIPDataType):
         """
         This method takes the dimension value of the array object and recursively converts the list into byte data
         """
+
         if dimension == 1:
             data = b''
             for element in list_data:
@@ -580,7 +640,19 @@ class CIPArray(CIPDataType):
         return self._list_representation
 
     def from_value(self, value):
-        self.data = self._recursive_array_to_data(self.array_dimensions, value)
+        if self.array_data_type == b'\xc1':
+            flat_list = flatten(self._list_representation)
+            data_int = 0
+            offset = 0
+            for item in flat_list:
+                if item:
+                    data_int = set_bit(data_int, offset)
+                else:
+                    data_int = clear_bit(data_int, offset)
+                offset = offset + 1
+            self.data = data_int.to_bytes(self.size, 'little')
+        else:
+            self.data = self._recursive_array_to_data(self.array_dimensions, value)
 
 
 # class CIPDataTypes:
